@@ -10,22 +10,40 @@ using Test
 using SparseArrays
 using LinearAlgebra
 using Random
-
+include("forbidproperties.jl")
+function test_map_and_map!(A, alloc_tests)
+    # --> test map entry point
+    fA = Array(A)
+    shapeA = size(A)
+    @test map(sin, A) == sparse(map(sin, fA))
+    @test map(cos, A) == sparse(map(cos, fA))
+    # --> test map! entry point
+    fX = copy(fA); X = similar(A)
+    map!(sin, X, A); X = similar(A) # warmup for @allocated
+    map!(sin, X, A); X = similar(A) # warmup for @allocated
+    if @allocated(map!(sin, X, A)) != 0 && alloc_tests
+        println(stderr, "A in test_map_and_map! is ", A)
+    end
+    X = similar(A)
+    @test @allocated(map!(sin, X, A)) == 0 || !alloc_tests
+    @test map!(sin, X, A) == sparse(map!(sin, fX, fA))
+    @test map!(cos, X, A) == sparse(map!(cos, fX, fA))
+    @test_throws DimensionMismatch map!(sin, X, spzeros((shapeA .- 1)...))
+end
 @testset "map[!] implementation specialized for a single (input) sparse vector/matrix" begin
     N, M = 10, 12
     for shapeA in ((N,), (N, M))
-        A = sprand(shapeA..., 0.4); fA = Array(A)
-        # --> test map entry point
-        @test map(sin, A) == sparse(map(sin, fA))
-        @test map(cos, A) == sparse(map(cos, fA))
-        # --> test map! entry point
-        fX = copy(fA); X = sparse(fX)
-        map!(sin, X, A); X = sparse(fX) # warmup for @allocated
-        @test (@allocated map!(sin, X, A)) == 0
-        @test map!(sin, X, A) == sparse(map!(sin, fX, fA))
-        @test map!(cos, X, A) == sparse(map!(cos, fX, fA))
-        @test_throws DimensionMismatch map!(sin, X, spzeros((shapeA .- 1)...))
+        A = sprand(shapeA..., 0.4)
+        nonzeros(A)[sin.(nonzeros(A)) .== 0] .= .0
+        nonzeros(A)[cos.(nonzeros(A)) .== 0] .= .0
+        A = dropzeros(A)
+        test_map_and_map!(A, false)
+        test_map_and_map!(A, false)
+        test_map_and_map!(A, true)
     end
+    # https://github.com/JuliaLang/julia/issues/37819
+    Z = spzeros(Float64, Int32, 50000, 50000)
+    @test isa(-Z, SparseMatrixCSC{Float64, Int32})
 end
 
 @testset "map[!] implementation specialized for a pair of (input) sparse vectors/matrices" begin
@@ -44,15 +62,18 @@ end
         # --> test map! entry point
         fX = map(+, fA, fB); X = sparse(fX)
         map!(+, X, A, B); X = sparse(fX) # warmup for @allocated
-        @test (@allocated map!(+, X, A, B)) == 0
+        @test (@allocated map!(+, X, A, B)) < 300
         @test map!(+, X, A, B) == sparse(map!(+, fX, fA, fB))
         fX = map(*, fA, fB); X = sparse(fX)
         map!(*, X, A, B); X = sparse(fX) # warmup for @allocated
-        @test (@allocated map!(*, X, A, B)) == 0
+        @test (@allocated map!(*, X, A, B)) < 300
         @test map!(*, X, A, B) == sparse(map!(*, fX, fA, fB))
         @test map!(f, X, A, B) == sparse(map!(f, fX, fA, fB))
         @test_throws DimensionMismatch map!(f, X, A, spzeros((shapeA .- 1)...))
     end
+    # https://github.com/JuliaLang/julia/issues/37819
+    Z = spzeros(Float64, Int32, 50000, 50000)
+    @test isa(Z + Z, SparseMatrixCSC{Float64, Int32})
 end
 
 @testset "map[!] implementation capable of handling >2 (input) sparse vectors/matrices" begin
@@ -71,11 +92,11 @@ end
         # --> test map! entry point
         fX = map(+, fA, fB, fC); X = sparse(fX)
         map!(+, X, A, B, C); X = sparse(fX) # warmup for @allocated
-        @test (@allocated map!(+, X, A, B, C)) == 0
+        @test (@allocated map!(+, X, A, B, C)) < 300
         @test map!(+, X, A, B, C) == sparse(map!(+, fX, fA, fB, fC))
         fX = map(*, fA, fB, fC); X = sparse(fX)
         map!(*, X, A, B, C); X = sparse(fX) # warmup for @allocated
-        @test (@allocated map!(*, X, A, B, C)) == 0
+        @test (@allocated map!(*, X, A, B, C)) < 300
         @test map!(*, X, A, B, C) == sparse(map!(*, fX, fA, fB, fC))
         @test map!(f, X, A, B, C) == sparse(map!(f, fX, fA, fB, fC))
         @test_throws DimensionMismatch map!(f, X, A, B, spzeros((shapeA .- 1)...))
@@ -118,12 +139,12 @@ end
         # --> test broadcast! entry point / zero-preserving op
         broadcast!(sin, fZ, fX); Z = sparse(fZ)
         broadcast!(sin, Z, X); Z = sparse(fZ) # warmup for @allocated
-        @test (@allocated broadcast!(sin, Z, X)) == 0
+        @test (@allocated broadcast!(sin, Z, X)) < 300
         @test broadcast!(sin, Z, X) == sparse(broadcast!(sin, fZ, fX))
         # --> test broadcast! entry point / not-zero-preserving op
         broadcast!(cos, fZ, fX); Z = sparse(fZ)
         broadcast!(cos, Z, X); Z = sparse(fZ) # warmup for @allocated
-        @test (@allocated broadcast!(cos, Z, X)) == 0
+        @test (@allocated broadcast!(cos, Z, X)) < 300
         @test broadcast!(cos, Z, X) == sparse(broadcast!(cos, fZ, fX))
         # --> test shape checks for broadcast! entry point
         # TODO strengthen this test, avoiding dependence on checking whether
@@ -142,12 +163,12 @@ end
         # --> test broadcast! entry point / zero-preserving op
         broadcast!(sin, fV, fX); V = sparse(fV)
         broadcast!(sin, V, X); V = sparse(fV) # warmup for @allocated
-        @test (@allocated broadcast!(sin, V, X)) == 0
+        @test (@allocated broadcast!(sin, V, X)) < 300
         @test broadcast!(sin, V, X) == sparse(broadcast!(sin, fV, fX))
         # --> test broadcast! entry point / not-zero-preserving
         broadcast!(cos, fV, fX); V = sparse(fV)
         broadcast!(cos, V, X); V = sparse(fV) # warmup for @allocated
-        @test (@allocated broadcast!(cos, V, X)) == 0
+        @test (@allocated broadcast!(cos, V, X)) < 300
         @test broadcast!(cos, V, X) == sparse(broadcast!(cos, fV, fX))
         # --> test shape checks for broadcast! entry point
         # TODO strengthen this test, avoiding dependence on checking whether
@@ -195,17 +216,17 @@ end
             # --> test broadcast! entry point / +-like zero-preserving op
             broadcast!(+, fZ, fX, fY); Z = sparse(fZ)
             broadcast!(+, Z, X, Y); Z = sparse(fZ) # warmup for @allocated
-            @test (@allocated broadcast!(+, Z, X, Y)) == 0
+            @test (@allocated broadcast!(+, Z, X, Y)) < 300
             @test broadcast!(+, Z, X, Y) == sparse(broadcast!(+, fZ, fX, fY))
             # --> test broadcast! entry point / *-like zero-preserving op
             broadcast!(*, fZ, fX, fY); Z = sparse(fZ)
             broadcast!(*, Z, X, Y); Z = sparse(fZ) # warmup for @allocated
-            @test (@allocated broadcast!(*, Z, X, Y)) == 0
+            @test (@allocated broadcast!(*, Z, X, Y)) < 300
             @test broadcast!(*, Z, X, Y) == sparse(broadcast!(*, fZ, fX, fY))
             # --> test broadcast! entry point / not zero-preserving op
             broadcast!(f, fZ, fX, fY); Z = sparse(fZ)
             broadcast!(f, Z, X, Y); Z = sparse(fZ) # warmup for @allocated
-            @test (@allocated broadcast!(f, Z, X, Y)) == 0
+            @test (@allocated broadcast!(f, Z, X, Y)) < 300
             @test broadcast!(f, Z, X, Y) == sparse(broadcast!(f, fZ, fX, fY))
             # --> test shape checks for both broadcast and broadcast! entry points
             # TODO strengthen this test, avoiding dependence on checking whether
@@ -258,17 +279,17 @@ end
             # --> test broadcast! entry point / +-like zero-preserving op
             fQ = broadcast(+, fX, fY, fZ); Q = sparse(fQ)
             broadcast!(+, Q, X, Y, Z); Q = sparse(fQ) # warmup for @allocated
-            @test (@allocated broadcast!(+, Q, X, Y, Z)) == 0
+            @test (@allocated broadcast!(+, Q, X, Y, Z)) < 300
             @test broadcast!(+, Q, X, Y, Z) == sparse(broadcast!(+, fQ, fX, fY, fZ))
             # --> test broadcast! entry point / *-like zero-preserving op
             fQ = broadcast(*, fX, fY, fZ); Q = sparse(fQ)
             broadcast!(*, Q, X, Y, Z); Q = sparse(fQ) # warmup for @allocated
-            @test (@allocated broadcast!(*, Q, X, Y, Z)) == 0
+            @test (@allocated broadcast!(*, Q, X, Y, Z)) < 300
             @test broadcast!(*, Q, X, Y, Z) == sparse(broadcast!(*, fQ, fX, fY, fZ))
             # --> test broadcast! entry point / not zero-preserving op
             fQ = broadcast(f, fX, fY, fZ); Q = sparse(fQ)
             broadcast!(f, Q, X, Y, Z); Q = sparse(fQ) # warmup for @allocated
-            @test (@allocated broadcast!(f, Q, X, Y, Z)) == 0
+            @test (@allocated broadcast!(f, Q, X, Y, Z)) < 300
             @test broadcast!(f, Q, X, Y, Z) == sparse(broadcast!(f, fQ, fX, fY, fZ))
             # --> test shape checks for both broadcast and broadcast! entry points
             # TODO strengthen this test, avoiding dependence on checking whether
@@ -342,11 +363,8 @@ end
             @test broadcast!(*, X, sparseargs...) == sparse(broadcast!(*, fX, denseargs...))
             @test isa(@inferred(broadcast!(*, X, sparseargs...)), SparseMatrixCSC{elT})
             X = sparse(fX) # reset / warmup for @allocated test
-            # It'd be nice for this to be zero, but there's currently some constant overhead
-            @test_broken (@allocated broadcast!(*, X, sparseargs...)) == 0
-            X = sparse(fX) # reset / warmup for @allocated test
             # And broadcasting over Transposes currently requires making a CSC copy, so we must account for that in the bounds
-            @test (@allocated broadcast!(*, X, sparseargs...)) <= (sum(x->isa(x, Transpose) ? @allocated(SparseMatrixCSC(x))+128 : 0, sparseargs) + 128)
+            @test (@allocated broadcast!(*, X, sparseargs...)) <= (sum(x->isa(x, Transpose) ? @allocated(SparseMatrixCSC(x)) + 128 : 0, sparseargs) + 128 + 900) # about zero to 3k bytes
         end
     end
     # test combinations at the limit of inference (eight arguments net)
@@ -366,9 +384,7 @@ end
         @test broadcast!(*, X, sparseargs...) == sparse(broadcast!(*, fX, denseargs...))
         @test isa(@inferred(broadcast!(*, X, sparseargs...)), SparseMatrixCSC{elT})
         X = sparse(fX) # reset / warmup for @allocated test
-        @test_broken (@allocated broadcast!(*, X, sparseargs...)) == 0
-        X = sparse(fX) # reset / warmup for @allocated test
-        @test (@allocated broadcast!(*, X, sparseargs...)) <= 128
+        @test (@allocated broadcast!(*, X, sparseargs...)) <= 900
     end
 end
 
@@ -624,7 +640,8 @@ end
     @test 2 .* ((1:5) .+ A) == 2:2:10
     @test 2 .* (A .+ (1:5)) == 2:2:10
 
-    @test Diagonal(spzeros(5)) \ view(rand(10), 1:5) == [Inf,Inf,Inf,Inf,Inf]
+    # lu(zeros(5,5)) throw SingularException, see #42343
+    @test_throws SingularException Diagonal(spzeros(5)) \ view(rand(10), 1:5)
 end
 
 @testset "Issue #27836" begin
@@ -635,7 +652,8 @@ end
     @test ((_, x) -> x).(Int, spzeros(3)) == spzeros(3)
     @test ((_, _, x) -> x).(Int, Int, spzeros(3)) == spzeros(3)
     @test ((_, _, _, x) -> x).(Int, Int, Int, spzeros(3)) == spzeros(3)
-    @test_broken ((_, _, _, _, x) -> x).(Int, Int, Int, Int, spzeros(3)) == spzeros(3)
+    @test ((_, _, _, _, x) -> x).(Int, Int, Int, Int, spzeros(3)) == spzeros(3)
+    @test_broken typeof(((_, _, _, _, x) -> x).(Int, Int, Int, Int, spzeros(3))) == typeof(spzeros(3))
 end
 
 using SparseArrays.HigherOrderFns: SparseVecStyle, SparseMatStyle
@@ -652,7 +670,7 @@ using SparseArrays.HigherOrderFns: SparseVecStyle, SparseMatStyle
     end
     @test err isa MethodError
     @test !occursin("is ambiguous", sprint(showerror, err))
-    @test occursin("no method matching _copy(::typeof(rand))", sprint(showerror, err))
+    @test err.f === SparseArrays.HigherOrderFns._copy
 end
 
 @testset "Sparse outer product, for type $T and vector $op" for
@@ -691,4 +709,71 @@ end
     @test SparseMatStyle(Val(3)) == Broadcast.DefaultArrayStyle{3}()
 end
 
+@testset "extrema" begin
+    n = 10
+    A = sprand(n, n, 0.2)
+    B = Array(A)
+    C = Array{Real}(undef, 0, 0)
+    x = sprand(n, 0.2)
+    y = Array(x)
+    z = Array{Real}(undef, 0)
+    f(x) = x^3
+    @test extrema(A) == extrema(B)
+    @test extrema(x) == extrema(y)
+    @test extrema(f, A) == extrema(f, B)
+    @test extrema(f, x) == extrema(f, y)
+    @test extrema(spzeros(n, n)) == (0.0, 0.0)
+    @test extrema(spzeros(n)) == (0.0, 0.0)
+    @test_throws "reducing over an empty" extrema(spzeros(0, 0))
+    @test_throws "reducing over an empty" extrema(spzeros(0))
+    @test extrema(sparse(ones(n, n))) == (1.0, 1.0)
+    @test extrema(sparse(ones(n))) == (1.0, 1.0)
+    @test extrema(A; dims=:) == extrema(B; dims=:)
+    @test extrema(A; dims=1) == extrema(B; dims=1)
+    @test extrema(A; dims=2) == extrema(B; dims=2)
+    @test extrema(A; dims=(1,2)) == extrema(B; dims=(1,2))
+    @test extrema(f, A; dims=1) == extrema(f, B; dims=1)
+    @test_throws "reducing over an empty" extrema(sparse(C); dims=1) == extrema(C; dims=1)
+    @test extrema(A; dims=[]) == extrema(B; dims=[])
+    @test extrema(x; dims=:) == extrema(y; dims=:)
+    @test extrema(x; dims=1) == extrema(y; dims=1)
+    @test extrema(f, x; dims=1) == extrema(f, y; dims=1)
+    @test_throws "reducing over an empty" extrema(sparse(z); dims=1)
+    @test extrema(x; dims=[]) == extrema(y; dims=[])
+end
+
+function test_extrema(a; dims_test = ((), 1, 2, (1,2), 3))
+    for dims in dims_test
+        vext = extrema(a; dims)
+        vmin, vmax = minimum(a; dims), maximum(a; dims)
+        @test all(x -> isequal(x[1], x[2:3]), zip(vext,vmin,vmax))
+    end
+end
+@testset "NaN test for sparse extrema" begin
+    for sz = (3, 10, 100)
+        A = sprand(sz, sz, 0.3)
+        A[rand(1:sz^2,sz)] .= NaN
+        test_extrema(A)
+        A = sprand(sz*sz, 0.3)
+        A[rand(1:sz^2,sz)] .= NaN
+        test_extrema(A; dims_test = ((), 1, 2))
+    end
+end
+
+@testset "issue #42670 - error in sparsevec outer product" begin
+    A = spzeros(Int, 4)
+    B = copy(A)
+    C = sparsevec([0 0 1 1 0 0])'
+    A[2] = 1
+    A[2] = 0
+    @test A * C == B * C == spzeros(Int, 4, 6)
+end
+
+@testset "issue #46337 - error in sparsevec map" begin
+    x = sparsevec([1], [1//1+0im])
+    @test inv.(x) == [1//1+0im]
+    y = spzeros(Int, 1)
+    @test y ./ x == y
+end
+
 end # module
